export default class ComponentMenu extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            isOpenFile: false,
            isOpenLog: false,
            isOpenRosTopics: false,
            anchorEl: null,
            is_recording: false,
            is_processing_recording: false,
        }

        document.onkeydown = (e) => {
            try {
                e = e || window.event;
                if (e.ctrlKey || e.metaKey) {
                    let c = e.which || e.keyCode;
                    switch (c) {
                        case 79:// Ctrl+O
                            e.preventDefault();
                            e.stopPropagation();
                            if(e.shiftKey) {
                                this.onOpenPLYandTXT();
                            } else {
                                this.onOpenProject();
                            }
                            break;
                        case 83:// Ctrl+S
                            e.preventDefault();
                            e.stopPropagation();
                            if(e.shiftKey) {
                                this.onSaveAsProject();
                            } else {
                                this.onSaveProject();
                            }
                            break;
                        case 69:// Ctrl+E
                            e.preventDefault();
                            e.stopPropagation();
                            this.onExportBag();
                            break;
                    }
                }
            } catch (e) {
                console.error(e);
            }
        };
        document.addEventListener('RECORDING_STATUS', this.onRecordingStatus)
    }

    onRecordingStatus = (e) => {
        this.setState({
            is_recording: e.detail.is_recording,
            is_processing_recording: false
        })
    }

    onSaveProject = async () => {
        this.setState({isOpenFile : false});
        let result = await eel.save_project()();
        if(result) {
            alert(result);
        } else {
            alert("Project saved successfully");
        }
    }

    onSaveAsProject = async () => {
        this.setState({isOpenFile : false});
        let result = await eel.save_as_project()();
        if(result) {
            alert(result);
        } else {
            alert("Project saved successfully");
        }
    }

    onOpenProject = async () => {
        this.setState({isOpenFile : false});
        let result = await eel.open_project()();
        if(result) {
            alert(result);
        } else {
            this.resetMapSize();
        }
    }

    onOpenPLYandTXT = () => {
        this.setState({isOpenFile : false});
        this.props.onDialogLoadPLYandTXT();
    }

    onConfigureRosTopics = () => {
        eel.get_ros_topics();
        this.setState({isOpenLog : false});
        this.props.onConfigureRosTopics();
    }

    onExportBag = () => {
        eel.get_ros_topics();
        this.setState({isOpenLog : false});
        eel.write_to_bag_file()();
        // this.props.onDialogExportBag();
    }

    onClearLogs = () => {
        eel.clear_logs();
        this.setState({isOpenLog : false});
    }

    resetMapSize = () => {
        $('a[data-val="auto"]').trigger("click");
    }

    render = () => {
        return (
            <div style={{position : "relative", width: "100%"}}>
                <Button
                    aria-controls={this.state.isOpenFile ? 'demo-customized-menu' : undefined}
                    aria-haspopup="true"
                    aria-expanded={this.state.isOpenFile ? 'true' : undefined}
                    variant="text"
                    disableElevation
                    onClick={(event)=> {
                        this.setState({
                            isOpenFile : !this.state.isOpenFile,
                            anchorEl: event.currentTarget
                        });
                    }}
                >
                    File
                </Button>
                <Button
                    aria-controls={this.state.isOpenLog ? 'demo-customized-menu' : undefined}
                    aria-haspopup="true"
                    aria-expanded={this.state.isOpenLog ? 'true' : undefined}
                    variant="text"
                    disableElevation
                    onClick={(event)=> {
                        this.setState({
                            isOpenLog : !this.state.isOpenLog,
                            anchorEl: event.currentTarget
                        });
                    }}
                >
                    Log
                </Button>
                <Button
                    id={'record-button'}
                    variant="text"
                    disableElevation
                    aria-controls={this.state.isOpenRecord ? 'demo-customized-menu' : undefined}
                    aria-haspopup="true"
                    aria-expanded={this.state.isOpenRecord ? 'true' : undefined}
                    onClick={(event)=> {
                        if(!this.state.is_processing_recording) {
                            if (this.state.is_recording) {
                                eel.stop_recording()();
                                this.setState({is_processing_recording: true});
                            } else {
                                eel.start_recording()();
                                this.setState({is_processing_recording: true});
                            }
                        }
                    }}
                >
                    {this.state.is_processing_recording && <span style={{ paddingRight: 10, paddingTop: 5 }}><CircularProgress size={20} /></span>}
                    {this.state.is_recording ? "Stop recording" : "Record screen"}
                </Button>

                <Menu
                    anchorEl={this.state.anchorEl}
                    anchorOrigin={{
                        vertical: 'bottom',
                        horizontal: 'right',
                    }}
                    transformOrigin={{
                        vertical: 'top',
                        horizontal: 'right',
                    }}
                    open={this.state.isOpenFile}
                    onClose={()=> {
                        this.setState({isOpenFile : false});
                    }}
                >
                    <MenuItem onClick={this.onOpenPLYandTXT} disableRipple>
                        <span className="material-icons">folder_open</span>
                        Open PLY and TXT.. (Ctrl+Shift+O)
                    </MenuItem>
                    <MenuItem onClick={this.onOpenProject} disableRipple>
                        <span className="material-icons">file_open</span>
                        Open Project.. (Ctrl+O)
                    </MenuItem>

                    <MenuItem onClick={this.onSaveProject} disableRipple>
                        <span className="material-icons">save</span>
                        Save Project.. (Ctrl+S)
                    </MenuItem>

                    <MenuItem onClick={this.onSaveAsProject} disableRipple>
                        <span className="material-icons">save</span>
                        Save As Project.. (Ctrl+Shift+S)
                    </MenuItem>

                    <Divider sx={{ my: 0.5 }} />
                    <MenuItem onClick={() => {
                        window.close();
                    }} disableRipple>
                        <span className="material-icons">close</span>
                        Exit
                    </MenuItem>
                </Menu>

                <Menu
                    anchorEl={this.state.anchorEl}
                    anchorOrigin={{
                        vertical: 'bottom',
                    }}
                    open={this.state.isOpenLog}
                    onClose={()=> {
                        this.setState({isOpenLog : false});
                    }}
                >
                    <MenuItem onClick={this.onConfigureRosTopics} disableRipple>
                        <span className="material-icons">settings_icon</span>
                        Configure ROS topics to show
                    </MenuItem>
                    <MenuItem onClick={this.onExportBag} disableRipple>
                        <span className="material-icons">file_upload_icon</span>
                        Export current topics to bag file
                    </MenuItem>

                    <MenuItem onClick={this.onClearLogs} disableRipple>
                        <span className="material-icons">delete_outline_icon</span>
                        Clear all logs history
                    </MenuItem>
                </Menu>
            </div>);
    }
}


