import ComponentLogLine from "./ComponentLogLine";

export default class ComponentPanelLogs extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            isLogsOpened: !!props.isOpened,
            currentPanel: 'logs', // logs, ros_topics
            logs: this.props.logs,
            scrollToBottom: true,
            is_searching: false,
            search_term: "",
            cur_highlight: null
        };
        document.addEventListener(UPDATE_STATE_LOGS, this.updateStateLogs);
        document.onkeydown = (e) => {
            try {
                e = e || window.event;
                if (e.ctrlKey || e.metaKey) {
                    let c = e.which || e.keyCode;
                    switch (c) {
                        case 70:// Ctrl+F
                            e.preventDefault();
                            e.stopPropagation();
                            this.onSearch();
                            $("#search_input").focus(); // Needs to b here to focus
                            break;
                    }
                }
            } catch (e) {
                console.error(e);
            }
        };
    }

    updateStateLogs = (event) => {
        try {
            this.setState({
                logs: event.detail
            });
        } catch (e) {
            alert(e.message);
        }
    }

    onTogglePanel = (pressedPanel) => {
        try {
            let currentPanel = pressedPanel

            let isLogsOpened = this.state.isLogsOpened;
            if(currentPanel === this.state.currentPanel || !isLogsOpened) {
                isLogsOpened = !this.state.isLogsOpened;
            }
            this.setState({
                isLogsOpened: isLogsOpened,
                currentPanel: currentPanel,
            });

            if (this.props.onTogglePanel) {
                setTimeout(() => {
                    this.props.onTogglePanel();
                }, 1);
            }
        } catch (e) {
            console.log(e);
        }
    }

    scrollToBottom = () => {
        try {
            let jEl = $("#panel-logs-feed");
            if(jEl.length) {
                jEl[0].scrollTop = 0;
            }
        } catch (e) {
            alert(e.message);
        }
    }

    scrollTo = (el) => {
        try {
            let jEl = $("#panel-logs-feed");
            if(jEl.length) {
                jEl[0].scrollTop = el[0].offsetTop;
            }
        } catch (e) {
            alert(e.message);
        }
    }

    onScrollLogs = (evt) => {
        try {
            let isAtBottom = false;
            let jEl = $("#panel-logs-feed");
            if(jEl.length && jEl[0].scrollTop > -1) {
                isAtBottom = true;
            }
            this.setState({
                scrollToBottom: isAtBottom
            })
        } catch (e) {
            console.log(e);
        }
    }

    onScrollToBottomLogs = (evt) => {
        try {
            this.setState({
                scrollToBottom: !this.state.scrollToBottom
            })
            this.scrollToBottom();
        } catch (e) {
            console.log(e);
        }
    }

    onSearch = () => {
        try {
            this.setState({
                is_searching: !this.state.is_searching,
                cur_highlight: null
            });
            if(!this.state.is_searching) {
                setTimeout(() => {
                    $("#search_input").focus().focusout(() => {
                        this.setState({
                            is_searching: false,
                            cur_highlight: null
                        });
                    });
                }, 20);
            }else{
            }
        } catch (e) {
            console.log(e);
        }
    }

    onSearchInput = (evt) => {
        try {
            if(evt.type === 'change') {
                this.setState({
                    search_term: evt.target.value
                })
                setTimeout(()=> {
                    this.focusNextSearch(true);
                }, 20);
            }else if(evt.type === 'keydown'){
                if(evt.keyCode === 13){
                    if(evt.shiftKey){
                        this.focusPrevSearch()
                    }else {
                        this.focusNextSearch();
                    }
                }
            }
        } catch (e) {
            console.log(e);
        }
    }

    focusNextSearch = (only_if_cur_invalid=false) => {
        if(only_if_cur_invalid && this.state.cur_highlight && this.state.cur_highlight.hasClass('highlighted')){
            return;
        }
        let scrolled = false;
        if(this.state.cur_highlight){
            let next = this.state.cur_highlight.nextAll('.log-line.highlighted')
            if(next.length){
                next = $(next[0]);
                this.setState({
                    cur_highlight: next
                })
                this.scrollTo(next);
                scrolled = true;
            }
        }
        if(!scrolled) {
            let highlighted = $('.log-line.highlighted');
            if (highlighted.length) {
                let next = $(highlighted[0]);
                this.setState({
                    cur_highlight: next
                })
                this.scrollTo(next);
                scrolled = true;
            }
        }
        if(!scrolled){
            this.setState({
                cur_highlight: null
            })
        }
    }

    focusPrevSearch = (only_if_cur_invalid=false) => {
        if(only_if_cur_invalid && this.state.cur_highlight && this.state.cur_highlight.hasClass('highlighted')){
            return;
        }
        let scrolled = false;
        if(this.state.cur_highlight){
            let next = this.state.cur_highlight.prevAll('.log-line.highlighted')
            if(next.length){
                next = $(next[0]);
                this.setState({
                    cur_highlight: next
                })
                this.scrollTo(next);
                scrolled = true;
            }
        }
        if(!scrolled) {
            let highlighted = $('.log-line.highlighted');
            if (highlighted.length) {
                let next = $(highlighted[highlighted.length-1]);
                this.setState({
                    cur_highlight: next
                })
                this.scrollTo(next);
                scrolled = true;
            }
        }
        if(!scrolled){
            this.setState({
                cur_highlight: null
            })
        }
    }

    render = () => {
        return (
            <div id="panel-logs">
                <div id="panel-logs-tabs">
                    <Tab
                        label="Log"
                        className={this.state.isLogsOpened && (this.state.currentPanel === "logs" ? "active" : "")}
                        onClick={() => {this.onTogglePanel("logs")}}
                    />
                    <Tab
                        label="Ros Topics"
                        className={this.state.isLogsOpened && (this.state.currentPanel === "ros_topics" ? "active": "")}
                        onClick={() => {this.onTogglePanel("ros_topics")}}
                    />
                </div>
                {this.state.isLogsOpened && this.state.logs &&
                    <div id="panel-logs-content">
                        <div id={"panel-logs-control-panel"}>
                            <p title={"Scroll to bottom and autoscroll"} id={"logs-scroll-to-bottom"} onClick={this.onScrollToBottomLogs}>
                                <svg xmlns="http://www.w3.org/2000/svg" viewBox={"0 0 48 48"}>
                                    <path d="M8 42v-3h32v3Zm16-6-9.7-9.7 2.15-2.15 6.05 6.05V6h3v24.2l5.85-5.85 2.15 2.15Z" fill={this.state.scrollToBottom? "white" : "gray"}/>
                                </svg>
                            </p>
                            <p title={"Find in logs"} id={"logs-search"} onClick={this.onSearch}>
                                <svg xmlns="http://www.w3.org/2000/svg" viewBox={"0 0 48 48"}>
                                    <path fill={this.state.is_searching? "white" : "gray"} d="M39.8 41.95 26.65 28.8q-1.5 1.3-3.5 2.025-2 .725-4.25.725-5.4 0-9.15-3.75T6 18.75q0-5.3 3.75-9.05 3.75-3.75 9.1-3.75 5.3 0 9.025 3.75 3.725 3.75 3.725 9.05 0 2.15-.7 4.15-.7 2-2.1 3.75L42 39.75Zm-20.95-13.4q4.05 0 6.9-2.875Q28.6 22.8 28.6 18.75t-2.85-6.925Q22.9 8.95 18.85 8.95q-4.1 0-6.975 2.875T9 18.75q0 4.05 2.875 6.925t6.975 2.875Z"/>
                                </svg>
                            </p>
                        </div>
                        {this.state.is_searching && (
                            <div id={"panel-logs-search"} >
                                <input id={"search_input"} onChange={this.onSearchInput} onKeyDown={this.onSearchInput} value={this.state.search_term}/>
                                <div id={"search_up"} className={'search_button'} onClick={() => {this.focusNextSearch()}}></div>
                                <div id={"search_down"} className={'search_button'} onClick={() => {this.focusPrevSearch()}}></div>
                            </div>
                        )}
                        <div id={"panel-logs-feed"} onScroll={this.onScrollLogs}>
                            {(this.state.currentPanel === "logs" ?
                                    this.state.logs.logs && [...this.state.logs.logs].sort((a, b)=> b.timestamp_unix - a.timestamp_unix).map((it, i) =>
                                        <ComponentLogLine
                                            key={`log-${it.timestamp_unix}-${it.source}-${this.state.logs.logs.length - i}`}
                                            timestamp={it.timestamp_unix}
                                            source={it.source}
                                            message={it.message}
                                            is_searching={this.state.is_searching}
                                            search_term={this.state.search_term}
                                            is_cur_highlight={this.state.cur_highlight && this.state.cur_highlight.attr('idx') === `${this.state.logs.logs.length - i}`}
                                            idx={this.state.logs.logs.length - i}
                                        />) :
                                    this.state.logs.logs_ros_topics && [...this.state.logs.logs_ros_topics].sort((a, b)=> b.timestamp_unix - a.timestamp_unix).map((it, i) =>
                                        <ComponentLogLine
                                            key={`topic-${it.timestamp_unix}-${it.topic_full_path}-${this.state.logs.logs_ros_topics.length - i}`}
                                            timestamp={it.timestamp_unix}
                                            source={it.topic_full_path}
                                            message={it.message}
                                            is_searching={this.state.is_searching}
                                            search_term={this.state.search_term}
                                            is_cur_highlight={this.state.cur_highlight && this.state.cur_highlight.attr('idx') === `${this.state.logs.logs.length - i}`}
                                            idx={this.state.logs.logs.length - i}
                                        />)
                            )}
                        </div>
                    </div>}
            </div>);
    }
}


