export default class DialogLoadPLYandTXT extends React.Component {

    constructor(props) {
        super(props);
        this.state = {
            filePathPLY: '',
            filePathTXT: '',
            isLoading: false,
            completed: 0,
            loading_real_progress: 0,
            loading_msg: ''
        };

        document.addEventListener(UPDATE_LOADING_PROGRESS, this.updateLoadingProgress);
    }

    updateLoadingProgress = (e) => {
        this.setState({
            loading_real_progress: e.detail.progress,
            completed: Math.min(100, this.state.completed + (e.detail.progress - this.state.completed) / 2),
            loading_msg: e.detail.message
        });
    }

    onClose = (event, reason) => {
        document.removeEventListener(UPDATE_LOADING_PROGRESS, this.updateLoadingProgress);

        if (reason && reason === "backdropClick") {
            return;
        }
        this.setState({
            filePathPLY: '',
            filePathTXT: '',
            isLoading: false
        });
        this.props.onClose();
    }

    onOpenPLY = async () => {
        let filePathPLY = await eel.open_filepath_ply()();
        this.setState({
            filePathPLY: filePathPLY
        });
    }

    onOpenTXT = async () => {
        let filePathTXT = await eel.open_filepath_txt()();
        this.setState({
            filePathTXT: filePathTXT
        });
    }

    onOk = async () => {
        try {
            if (this.state.isLoading) {
                return;
            }
            if (this.state.filePathPLY.length > 0 && this.state.filePathTXT.length > 0) {
                this.setState({
                    isLoading: true
                });
                setInterval(() => this.progress(), 500)
                let result = await eel.load_ply_and_txt()()
                if (result === '') {
                    this.onClose();
                } else {
                    alert(result);
                    this.setState({
                        isLoading: false
                    });
                }
                $('a[data-val="auto"]').trigger("click");
            } else {
                alert('Please select PLY and TXT file')
            }
        } catch (e) {
            alert(e);
            this.setState({
                isLoading: false
            });
        }
    }

    progress = () => {
        const {completed} = this.state;
        let diff = (this.state.loading_real_progress - completed) / 3;
        if(diff > 0) {
            if(this.state.loading_real_progress === 100) {
                diff *= 3;
            }
            this.setState({
                completed: Math.min(100, completed + diff)
            });
        }
    };

    render = () => {

        const style = {
            width: 400,
            backgroundColor: "#f3f3f3",
            border: '2px solid #000',
            borderTop: 0,
            p: 4,
            borderRadius: "0 0 18px 18px",
        };

        const contain_style = {
            position: 'absolute',
            top: '50%',
            transform: 'translate(0, -50%)',
        }

        const title_bar_style = {
            fontSize: "102%",
            fontFamily: 'Space Grotesk',
            fontWeight: "600",
            letterSpacing: "0.5px",
            lineHeight: "25px",
            textAlign: "center",
            width: 400,
            backgroundColor: "#fff",
            border: '2px solid #000',
            borderBottom: 0,
            p: 1,
            borderRadius: "18px 18px 0 0",
        }

        const load_btn_style = {
            fontSize: "102%",
            padding: "6%",
            width: "100%"
        }

        const progress_bar_style = {
            marginTop: "2px",
            padding: "5px",
            height: "2px",
            border: "6px solid #fff",
            borderRadius: "18px",
        }

        const loading_style = {
            color: "#000",
            fontWeight: "600",
            letterSpacing: "0.5px",
        }

        return (
            <Modal
                open={this.props.isOpen}
                onClose={this.onClose}
            >
                <Grid container spacing={0} alignItems="center" direction="column" style={contain_style}>
                    <Grid item sx={title_bar_style}>Load files</Grid>
                    <Grid item sx={style}>

                        {!this.state.filePathPLY ?
                            <div>
                                <Button variant="contained" style={load_btn_style} onClick={this.onOpenPLY}>Load PLY
                                    file..</Button><br/>
                            </div> :
                            <div style={{paddingBottom: "20px"}}>
                                <div style={{color: "#E4701E", fontWeight: "600"}}>PLY</div>
                                <div style={loading_style}>{this.state.filePathPLY}</div>
                            </div>
                        }

                        {!this.state.filePathTXT ?
                            <div>
                                <Button variant="contained" style={load_btn_style} onClick={this.onOpenTXT}>Load TXT
                                    file..</Button><br/>
                            </div> :
                            <div style={{paddingBottom: "20px"}}>
                                <div style={{color: "#E4701E", fontWeight: "600"}}>TXT</div>
                                <div style={loading_style}>{this.state.filePathTXT}</div>
                            </div>
                        }

                        {this.state.isLoading &&
                            <div style={{color: "#E4701E", textAlign: "center", paddingTop: "20px"}}>
                                <span style={loading_style}>Loading...</span><br/>
                                <span style={{...loading_style, color: 'gray'}}>{this.state.loading_msg}</span>
                                <LinearProgress variant="determinate" value={this.state.completed}
                                                style={progress_bar_style} color="inherit"/>
                            </div>
                        }

                        <div style={{paddingTop: "20px"}}>
                            <Button
                                variant="action"
                                onClick={this.onClose}
                            >
                                Cancel
                            </Button>
                            <Button
                                variant="action"
                                onClick={this.onOk}
                            >
                                Ok
                            </Button>
                        </div>
                    </Grid>
                </Grid>
            </Modal>
        );
    }
}