import ComponentMenu from '../components/ComponentMenu';
import ComponentMap from '../components/ComponentMap';
import ComponentPanelLogs from '../components/ComponentPanelLogs';
import DialogChangePoiName from "./DialogChangePoiName";
import DialogSavePOI from "./DialogSavePOI";
import DialogDeletePOIs from "./DialogDeletePOIs";
import DialogRosTopics from "./DialogRosTopics";
import DialogFlightPlan from "./DialogFlightPlan";
import DialogLoadPLYandTXT from "./DialogLoadPLYandTXT";
import ComponentPanelLeft from "../components/ComponentPanelLeft";
import ComponentPanelStatusBottom from "../components/ComponentPanelStatusBottom";

export default class DialogMain extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            project: false,
            isDialogLoadPLYandTXT: false,
            isDialogLoadRosTopics: false,
            isDialogFlightPlan: false,
            isDialogChangeNameOpen: false,
            isDialogSavePOI: false,
            isDialogDeletePOI: false,
            clearAllPOIs: false,
            rename_poi_uuid: "",
            rename_poi_name: "",
            isFlightPlanExecuting: false,
            exportBag: false
        };

        this.refMap = null;

        document.addEventListener(UPDATE_STATE_FRONTEND, this.updateStateFrontend);
        document.addEventListener(OPEN_NEW_FILE, this.openNewFile);

        document.addEventListener(POI_FLY_TO_FINISHED, this.poiFlyToFinished);
        document.addEventListener(TAKE_OFF_FINISHED, this.takeOffFinished);
        document.addEventListener(LAND_FINISHED, this.landFinished);

        $(window).resize(this.onWindowResize);
    }

    openNewFile = () => {
        this.setState({
        });
    }
    onWindowResize = (event) => {
        if (this.refMap) {
            this.refMap.onResize();
        }
    }

    updateStateFrontend = (event) => {
        try {
            this.setState({
                project: event.detail
            });
        } catch (e) {
            alert(e.message);
        }
    }

    componentDidMount = () => {
        eel.initialize();
    }

    poiFlyToFinished = () => {
        if(this.state.isFlightPlanExecuting) {
            this.onFlightPlanUpdateNextStep();
        }
    }

    takeOffFinished = () => {
        if(this.state.isFlightPlanExecuting) {
            this.onFlightPlanUpdateNextStep();
        }
    }

    landFinished = () => {
        if(this.state.isFlightPlanExecuting) {
            this.onFlightPlanUpdateNextStep();
        }
    }

    onFlightPlanDelayOrNextStep = () => {

        if(this.state.project) {
            //Initially wait delay before starting exextuting plan
            if(this.state.project.flight_plan.current_delay_before_execution_sec < this.state.project.flight_plan.delay_before_execution_sec) {
                eel.add_log("INFO", `Delay ${this.state.project.flight_plan.current_delay_before_execution_sec} / ${this.state.project.flight_plan.delay_before_execution_sec} sec.`);
                setTimeout(() => {
                    this.state.project.flight_plan.current_delay_before_execution_sec++;
                    this.setState({
                       project: this.state.project
                    });
                    this.onFlightPlanDelayOrNextStep();
                }, 1000);
            } else {
                eel.add_log("INFO", `Starting execution of flight plan.`);
                // Delay completed
                // Process next POIs in flight plan
                this.onFlightPlanUpdateNextStep();
            }

        }
    }

    onFlightPlanUpdateNextStep = () => {
        if(this.state.project) {

            for(let step of this.state.project.flight_plan.steps) {
                if(step.step_status === "Executing") {
                    step.step_status = "Completed";
                    break;
                }
            }

            let last_step_included_in_loop = _.findLast(
                this.state.project.flight_plan.steps,
                (step) => step.is_included_in_loop
            );
            if(last_step_included_in_loop) {
                if(last_step_included_in_loop.step_status === "Completed") {
                    this.state.project.flight_plan.current_loop_count++;

                    if(this.state.project.flight_plan.current_loop_count < this.state.project.flight_plan.repeat_count) {
                        for(let step of this.state.project.flight_plan.steps) {
                            if(step.is_included_in_loop) {
                                step.step_status = "Pending";
                            }
                        }
                        this.setState({
                            project: {...this.state.project}
                        });
                        this.onFlightPlanUpdateNextStep();
                        return;
                    }
                }
            }

             for(let step of this.state.project.flight_plan.steps) {
                if(step.step_status === "Pending") {
                    step.step_status = "Executing";
                    if(step.step_type === "POI") {
                        let poi = this.state.project.map_layers.layer_pois.pois.find(poi => poi.uuid === step.poi_uuid);
                        eel.adc_fly_to_poi(poi.name, poi.x, poi.y);
                    } else if(step.step_type === "TakeOff") {
                        eel.adc_take_off();
                    } else if (step.step_type === "Land") {
                        eel.adc_land();
                    }
                    break;
                }
            }

            let last_step = _.last(
                this.state.project.flight_plan.steps
            );
            if(last_step.step_status === "Completed") {
                this.setState({
                    isFlightPlanExecuting: false
                });
            }

            this.setState({
                project: {...this.state.project}
            });
        }
    }

    render = () => {
        if (!this.state.project) {
            return (<div></div>); //before loading
        }

        return (<div className="panels">
            <div className="panel-main">
                <div id="panel-menu-bar">
                    <ComponentMenu
                        onDialogLoadPLYandTXT={() => {
                            this.setState({isDialogLoadPLYandTXT: true});
                        }}
                        onConfigureRosTopics={() => {
                            this.setState({isDialogLoadRosTopics: true, exportBag: false});
                        }}
                        onDialogExportBag={() => {
                            this.setState({isDialogLoadRosTopics: true, exportBag: true});
                        }}
                    />
                </div>
                <div id="panel-main-plot">
                    <ComponentMap
                        ref={(ref) => this.refMap = ref}
                        project={this.state.project}
                    />
                </div>
                <ComponentPanelLogs
                    logs={this.state.project.logs}
                    isOpened={true}
                    onTogglePanel={() => {
                        if (this.refMap) {
                            this.refMap.onResize();
                        }
                    }}
                />
                {/* For now remove bottom status bar as we do not have those ROS topics available <ComponentPanelStatusBottom /> */}
            </div>
            <ComponentPanelLeft
                project={this.state.project}
                onStopFlightPlanExecution={() => {
                    this.setState({
                        isFlightPlanExecuting: false
                    });
                }}
                onDialogLoadFlightPlan={() => {
                    this.setState({isDialogFlightPlan: true});
                }}
                onDialogSavePOI={() => {
                    this.setState({isDialogSavePOI: true});
                }}
                onDialogDeletePOI={() => {
                    this.setState({isDialogDeletePOI: true});
                }}

                onDialogClearAllPOIs={() => {
                    this.setState({isDialogDeletePOI: true, clearAllPOIs: true});
                }}

                onUpdateProject={(project) => {
                    this.setState({
                        project: {...project}
                    });
                    eel.update_view_settings_json(JSON.stringify(this.state.project.view_settings));
                }}
            />
            {this.state.isDialogLoadPLYandTXT && <DialogLoadPLYandTXT isOpen={this.state.isDialogLoadPLYandTXT} onClose={() => {
                this.setState({isDialogLoadPLYandTXT: false});
            }}/>}
            {this.state.isDialogFlightPlan && <DialogFlightPlan
                isOpen={this.state.isDialogFlightPlan}
                project={this.state.project}
                isFlightPlanExecuting={this.state.isFlightPlanExecuting}
                onStopFlightPlan={() => {
                    eel.adc_stop();
                    this.setState({
                        isFlightPlanExecuting: false
                    });
                }}
                onExecutePlan={() => {
                    this.setState({isFlightPlanExecuting: true});
                    this.onFlightPlanDelayOrNextStep();
                }}
                onClose={() => {
                    this.setState({isDialogFlightPlan: false})
                }}
                onChangeNameOpen={(name, uuid) => this.setState({
                    isDialogChangeNameOpen: true,
                    rename_poi_name: name,
                    rename_poi_uuid: uuid
                })}
            />}
            {this.state.isDialogChangeNameOpen && <DialogChangePoiName
                poiName={this.state.rename_poi_name}
                poiUUID={this.state.rename_poi_uuid}
                isOpen={this.state.isDialogChangeNameOpen} onClose={() => {
                this.setState({isDialogChangeNameOpen: false})
            }}/>}
            {this.state.isDialogSavePOI && <DialogSavePOI
                project={this.state.project}
                isOpen={this.state.isDialogSavePOI} onClose={() => {
                this.setState({isDialogSavePOI: false})
            }}/>}
            {this.state.isDialogDeletePOI && <DialogDeletePOIs
                clearAllPOIs={this.state.clearAllPOIs}
                isOpen={this.state.isDialogDeletePOI} onClose={() => {
                this.setState({isDialogDeletePOI: false, clearAllPOIs: false})
            }}/>}
            {this.state.isDialogLoadRosTopics && <DialogRosTopics
                project={this.state.project}
                exportBag={this.state.exportBag}
                isOpen={this.state.isDialogLoadRosTopics}
                onClose={() => {
                    this.setState({isDialogLoadRosTopics: false})
                }}/>}
        </div>);
    }
}

