import ComponentRosNode from "../components/ComponentRosNode";

export default class DialogRosTopics extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            ros_topics: _.cloneDeep(props.project.ros_topics)
        };
    };

    componentDidMount = () => {

    }

    onClose = () => {
        this.props.onClose();
    }

    onApply = () => {
        eel.update_ros_topics_json(JSON.stringify(this.state.ros_topics))();
        this.onClose();
    }

    onExport = () => {
        eel.write_to_bag_file(JSON.stringify(this.state.ros_topics))();
        this.onClose();
    }

    setSelectedTopicAndChildren = (ros_topic, is_selected) =>
    {
        ros_topic.is_selected = is_selected;
        for(let sub_topic of ros_topic.children_topics) {
            this.setSelectedTopicAndChildren(sub_topic, is_selected);
        }
    }

    onToggleAll = async () => {
        let ros_topics = _.cloneDeep(this.state.ros_topics);
        let topics_selected = await eel.get_ros_topics_selected_from_all_top_topics(ros_topics, true)();

        if(topics_selected.length) {
            // if any topic selected then unselect all
            for(let ros_topic of ros_topics.ros_topics) {
                this.setSelectedTopicAndChildren(ros_topic, false);
            }
        }
        else
        {
            for(let ros_topic of ros_topics.ros_topics) {
                this.setSelectedTopicAndChildren(ros_topic, true);
            }
        }
        this.setState({ros_topics: ros_topics});
    }

    render = () => {
        const style = {
            width: 800,
            backgroundColor: "#f3f3f3",
            border: '2px solid #000',
            borderTop: 0,
            p: 4,
            borderRadius: "0 0 18px 18px",
        };

        const contain_style = {
            position: 'absolute',
            top: '50%',
            left: '50%',
            width: "auto",
            transform: 'translate(-50%, -50%)',
        }

        const title_bar_style = {
            fontSize: "102%",
            fontFamily: 'Space Grotesk',
            fontWeight: "600",
            letterSpacing: "0.5px",
            lineHeight: "25px",
            textAlign: "center",
            width: 800,
            backgroundColor: "#fff",
            border: '2px solid #000',
            borderBottom: 0,
            p: 1,
            borderRadius: "18px 18px 0 0",
        }

        const btn_style = {
            padding: "3%",
            width: "20%"
        }

        return (
            <Modal
                open={this.props.isOpen}
                onClose={this.onClose}>
                <Grid container spacing={0} alignItems="center" direction="column" style={contain_style}>
                    <Grid item sx={title_bar_style}>{this.props.exportBag? 'Export Bag file' : 'Configure ROS Topics to show'}</Grid>
                    <Grid item sx={style}>
                        <div style={{marginBottom: "10px"}}>{this.props.exportBag? 'Select topics to export to Bag file' : 'Topics to include'}</div>
                        <div style={{border: "2px solid #000", height: "400px", overflow: "auto"}}>
                            {this.state.ros_topics.ros_topics.map((topic, i) =>
                                <ComponentRosNode
                                    key={`${topic.full_path}-${i}`}
                                    ros_topic={topic}
                                    sub_topic_space={0}
                                    addTopic={this.addTopic}
                                    removeTopic={this.removeTopic} />
                            )}
                        </div>
                        <Button variant="action" style={{...btn_style, marginRight: "35%"}} onClick={this.onToggleAll}>Toggle All</Button>
                        <Button variant="action" style={btn_style} onClick={this.onClose}>Cancel</Button>
                        {this.props.exportBag ?
                            <Button variant="action" style={btn_style} onClick={this.onExport}>Export</Button>
                            :
                            <Button variant="action" style={btn_style} onClick={this.onApply}>Apply</Button>
                        }
                    </Grid>
                </Grid>
            </Modal>
        )
    }
}