import DialogWarning from "./DialogWarning";

export default class DialogSavePOI extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            poiName: "",
            poiUUID: "",
            isDialogWarning: false,
            message: ""
        };
    };

    onClose = () => {
        this.setState({
            poiName: ""
        });
        this.props.onClose();
    }

    onOk = () => {
        if (this.state.poiName) {
            let pois = [...this.props.project.map_layers.layer_pois.pois];
            if (pois.length) {
                let found = false;
                for (let poi of pois) {
                    if (poi.name === this.state.poiName) {
                        found = true;
                        break;
                    }
                }
                //Show same name warning popup if found
                if (found) {
                    this.setState({
                        isDialogWarning: true,
                        message: "Cannot have duplicate POI label!"
                    });
                } else {
                    document.dispatchEvent(new CustomEvent(POI_SAVE_ACTIVE, {detail: this.state.poiName}));
                    this.onClose();
                }
            } else {
                document.dispatchEvent(new CustomEvent(POI_SAVE_ACTIVE, {detail: this.state.poiName}));
                this.onClose();
            }
        } else {
            //Show empty name warning popup
            this.setState({
                isDialogWarning: true,
                message: "POI label cannot be empty!"
            });
        }
    }

    render = () => {
        const style = {
            width: 450,
            backgroundColor: "#f3f3f3",
            border: '2px solid #000',
            borderTop: 0,
            p: 4,
            borderRadius: "0 0 18px 18px",
        };

        const contain_style = {
            position: 'absolute',
            top: '50%',
            left: '50%',
            width: "auto",
            transform: 'translate(-50%, -50%)',
        }

        const title_bar_style = {
            fontSize: "102%",
            fontFamily: 'Space Grotesk',
            fontWeight: "600",
            letterSpacing: "0.5px",
            lineHeight: "25px",
            textAlign: "center",
            width: 450,
            backgroundColor: "#fff",
            border: '2px solid #000',
            borderBottom: 0,
            p: 1,
            borderRadius: "18px 18px 0 0",
        }

        return (
            <div>
                <Modal
                    open={this.props.isOpen}
                    onClose={this.onClose}>
                    <Grid container spacing={0} alignItems="center" direction="column" style={contain_style}>
                        <Grid item sx={title_bar_style}>Save POI</Grid>
                        <Grid item sx={style}>
                            <div style={{display: "flex", flexDirection: "row"}}>
                                <div style={{fontSize: "105%", alignSelf: "center"}}>Enter POI Label:</div>
                                <TextField variant="outlined"
                                           value={this.state.poiName}
                                           onChange={(e) => this.setState({poiName: e.target.value})}
                                           label={this.props.poiName}
                                           sx={{
                                                backgroundColor: '#fff',
                                                border: "1px solid #D5DADC",
                                                borderRadius: "5px",
                                                margin: "0 10px",
                                                width: "60%",
                                            }}/>
                            </div>
                            <Button variant="action" onClick={this.onClose}>Cancel</Button>
                            <Button variant="action" onClick={this.onOk}>Ok</Button>
                        </Grid>
                    </Grid>
                </Modal>
                <DialogWarning
                    message={this.state.message}
                    isOpen={this.state.isDialogWarning} onClose={() => {
                    this.setState({isDialogWarning: false})
                }}/>
            </div>
        )
    }
}